import distro
import os

from .typehints import PortalEnv


def setup_portal_environment(development=False, development_root='') -> PortalEnv:
    """
    Set environment variables for DrawBridgeOS Portal.
    """
    distro_id = distro.id()

    GENERAL_SERVER = os.path.exists('/etc/compass/settings.conf')

    if development and development_root:
        ROOT_HOME = PERSIST_DIR = development_root
    else:
        ROOT_HOME = PERSIST_DIR = '/opt/sap/logcabin_portal'

    IS_DRAWBRIDGE_OS = False
    INIT_FRAMEWORK = 'systemd'
    CODE_NAME = 'logcabin'
    BACKUP_DIR = '/opt/pgbackups'
    ETC_DIR = '/etc'
    CONF_DIR = f'{ROOT_HOME}/config'
    PROJECT_ROOT = f'{ROOT_HOME}/logcabin'
    APPS_DIR = '/opt/sap'
    PORTAL_CATEGORY_DIR = f'{ROOT_HOME}/categories'
    REDWOOD_LOG_DIR = '/var/log/redwood'
    SCRIPTS_DIR = '/opt/sap/scripts'
    TLS_DIR = f'{ROOT_HOME}/config'
    DALMATIAN_DIR = f'{APPS_DIR}/dalmatian'
    DNS_DIR = f'{APPS_DIR}/powerdns'

    if is_clearos(distro_id):
        INIT_FRAMEWORK = 'supervisord'

    elif not GENERAL_SERVER and is_drawbridge_os(distro_id):
        CODE_NAME = 'drawbridge'
        IS_DRAWBRIDGE_OS = True
        ROOT_HOME = PERSIST_DIR = SCRIPTS_DIR = f'{ETC_DIR}/drawbridge_portal'
        APPS_DIR = ETC_DIR
        BACKUP_DIR = '/usr/lib/live/mount/persistence/drawbridgeos/opt/pgbackups'
        CONF_DIR = f'{ROOT_HOME}/config'
        PROJECT_ROOT = f'{ROOT_HOME}/drawbridge'
        PORTAL_CATEGORY_DIR = f'{ROOT_HOME}/categories'
        TLS_DIR = f'{ROOT_HOME}/tls'
        DNS_DIR = f'{ROOT_HOME}/dns'

    if GENERAL_SERVER:
        PERSIST_DIR = CONF_DIR = '/etc/compass'
        PROJECT_ROOT = ROOT_HOME = os.getcwd()

    environment_vars: PortalEnv = {
        'INIT_FRAMEWORK': INIT_FRAMEWORK,
        'IS_DRAWBRIDGE_OS': IS_DRAWBRIDGE_OS,
        'ROOT_HOME': ROOT_HOME,
        'PORTAL_CODE_NAME': CODE_NAME,
        'PROJECT_ROOT': PROJECT_ROOT,
        'CONF_DIR': CONF_DIR,
        'BACKUP_DIR': BACKUP_DIR,
        'DALMATIAN_DIR': DALMATIAN_DIR,
        'DALMATIAN_LOG_DB_DIR': f'{DALMATIAN_DIR}/log_databases',
        'DALMATIAN_MAXMIND_DB_DIR': f'{DALMATIAN_DIR}/maxminddb',
        'DNS_DIR': DNS_DIR,
        'FIREWALL_DIR': f'{APPS_DIR}/firewall',
        'PERSIST_DIR': PERSIST_DIR,
        'PORTAL_CATEGORY_DIR': PORTAL_CATEGORY_DIR,
        'REDWOOD_CATEGORY_DIR': f'{ETC_DIR}/redwood/categories',
        'REDWOOD_CONTENTMODS_DIR': f'{ETC_DIR}/redwood/contentmods',
        'REDWOOD_LOG_DIR': REDWOOD_LOG_DIR,
        'SCRIPTS_DIR': SCRIPTS_DIR,
        'TLS_DIR': TLS_DIR,
        'CLARION_DIR': f'{ETC_DIR}/clarion',
        'PASSAGEWAY_DIR': f'{ETC_DIR}/passageway',
    }

    os.environ.update({k: str(v) for k, v in environment_vars.items() if v})

    return environment_vars


# ---------------------------------------------------------------
def is_drawbridge_os(distro_id='') -> bool:
    """Is this system running DrawBridge OS?"""
    distro_id = distro_id or distro.id()
    return distro_id.lower() in ('drawbridgeos', 'vyos')


# ---------------------------------------------------------------
def is_clearos(distro_id='') -> bool:
    """Is this system running ClearOS?"""
    distro_id = distro_id or distro.id()
    return distro_id.lower() == 'clearos'


# ---------------------------------------------------------------
def get_distro() -> str:
    """Get linux distribution of host system"""

    # ('ClearOS Community', '6.7.0', 'Final')
    # ('ClearOS', '7.2.0', 'Final')
    release = distro.linux_distribution()
    os_platform = release[0].split()[0].lower()
    major_version = release[1].split('.')[0]

    if release[0].lower().startswith('centos') and release[1].startswith('5'):
        os_platform = 'clearos'
        major_version = '5'

    return f'{os_platform}{major_version}'


__all__ = (
    'get_distro',
    'setup_portal_environment',
    'is_clearos',
    'is_drawbridge_os',
)
